#coding:utf=8

import os,re

"""
Author: Zhu Sitao
Date: 2019-4-3
Description: The BED format consists of one line per feature, each containing 3-12 columns of data,plus optional track definition lines.
 BED 文件(Browser Extensible Data)格式是ucsc 的genome browser的一个格式 ,提供了一种灵活的方式来定义的数据行，以用来描述注释信息.
 BED文件结构：
-------------------------------------------------------------必须有以下3列------------------------------------------------------------------------
    chrom :即染色体号
    chromStart :即feature在染色体上起始位置 。在染色体上最左端坐标是0
    chromEnd :即feature在染色体上的终止位置。例如一个染色体前100个碱基定义为chromStart=0,  chromEnd=100, 跨度为 0-99.
----------------------------------------------------------------可选9列-------------------------------------------------------------------------------
    name ：feature的名字 ，在基因组浏览器左边显示；
    score ：在基因组浏览器中显示的灰度设定，值介于0-1000；
    strand ：定义链的方向，‘‘+” 或者”-”
    thickStart ：起始位置(例如，基因起始编码位置）
    thickEnd ：终止位置（例如：基因终止编码位置）　
    itemRGB ：是一个RGB值的形式, R, G, B (eg. 255, 0,0), 如果itemRgb设置为‘On”, 这个RBG值将决定数据的显示的颜色。
    blockCount ：BED行中的block数目，也就是外显子数目
    blockSize：用逗号分割的外显子的大小, 这个item的数目对应于BlockCount的数目
    blockStarts ：用逗号分割的列表, 所有外显子的起始位置，数目也与blockCount数目对应
"""

class bedParse(object):
    """ a class for one line of a bed file """
    def __init__(self,chrom,start,end,name=None,score=None,strand=None,thickStart=None,thickEnd=None,itemRgb=None,blockCount=None,blockSize=None,blockStarts=None):
        self.chrom = chrom
        self.start = start
        self.end = end
        self.name = name
        self.score = score
        self.strand = strand
        self.thickStart = thickStart
        self.thickEnd = thickEnd
        self.itemRgb = itemRgb
        self.blockCount = blockCount
        self.blockSize = blockSize
        self.blockStarts = blockStarts

    def length(self):
        """get length of a line """
        return self.end - self.start


class bed(object):
    """ a bed class """
    def __init__(self,bed_path):
        self.bedPath = bed_path
        self.basename = os.path.basename(bed_path)
        self.dirname = os.path.dirname(bed_path)
    def readBed(self):
        """ a method for read bed file """
        for line in open(self.bedPath,'r').readline():
            yield line.strip()



            





import re,os

"""
Author :Zhu Sitao
Date: 2018-3-23
Dest: read bam file by samtools ;samtolls should be installed in your ~/.bashrc
      python version is 3.6
"""


class Bam(object):
	""" a readBam class """
	# class global variable , each funcation could use the variable
	patternAnnotation = re.compile(r'^@')

	def __init__(self,filePath):
		self.path = filePath
		self.sam = self.readBam()

	def readBam(self):
		""" Read bam from samtools view """
		samtools = os.popen('which samtools').read().strip()
		sam = os.popen('%s view -h %s' % (samtools,self.path)).read()
		iterm = sam.split("\n")
		itermFilter = [x for x in iterm if x !='']
		return itermFilter
		#for i in iterm:
		#	yield i

	def bamHeader(self):
		""" Return bam header to stdout """
		for line in self.sam:
			if self.patternAnnotation.match(line) and line != '':
				print (line)
	def mappingRate(self):
		""" Return bam mapping rate """
		clean_reads, clean_bases = 0, 0
		mapped_reads, mapped_bases = 0, 0
		mapping_rate, uniq_rate = 0, 0
		dup_reads, dup_rate = 0, 0
		mismatch_bases, mismatch_rate = 0, 0
		uniq_reads, uniq_bases = 0, 0
		mappedPattern = re.compile(r'XC:i:(\d+)')
		uniqPattern = re.compile(r'X0:i:(\d+)')
		mismatchPattern1 = re.compile(r'XM:i:(\d+)')
		mismatchPattern2 = re.compile(r'MD:Z:(\S+)')

		bamfh = os.popen("%s view %s" % (self.samtools, bam))
		for line in bamfh.readlines():
			tmp = line.strip().split()
			flag = int(tmp[1])
			if (flag & 0x100):
				continue
			else:
				clean_reads += 1
				clean_bases += len(tmp[9])
				if not (flag & 0x4):
					mapped_reads += 1
					if mappedPattern.search(line):
						mapped_bases += int(mappedPattern.search(line).group(1))
					else:
						mapped_bases += len(tmp[9])
					if flag & 0x400 == 0:
						if uniqPattern.search(line):
							if int(uniqPattern.search(line).group(1)) == 1:
								uniq_reads += 1
							uniq_bases += len(tmp[9])
						elif int(tmp[4]) > 0:
							uniq_reads += 1
							uniq_bases += len(tmp[9])
						if mismatchPattern1.search(line):
							mismatch_bases += int(mismatchPattern1.search(line).group(1))
						elif mismatchPattern2.search(line):
							mismatch_bases += self.CoutMismathNo(line)
					else:
						dup_reads += 1
		mismatch_rate = mismatch_bases / mapped_bases
		mapping_rate = mapped_reads / clean_reads
		dup_rate = dup_reads / mapped_reads
		uniq_rate = uniq_reads / mapped_reads
		name = os.path.basename(bam)
		sample = name.split('.')[0]
		print("Sample\t%s\n" % (sample))
		print("Clean reads\t%s\n" % (clean_reads))
		print("Clean bases (Mb)\t%.2f\n" % (clean_bases / 1000000))
		print("Mapping rate (%%)\t%.2f\n" % (100 * mapping_rate))
		print("Unique rate (%%)\t%.2f\n" % (100 * uniq_rate))
		print("Duplicate rate (%%)\t%.2f\n" % (100 * dup_rate))
		print("Mismatch rate (%%)\t%.2f\n" % (100 * mismatch_rate))

	def readLength(self):
		""" Read length """
		for line in self.sam:
			if not self.patternAnnotation.match(line) and line != '':
				readLength = len(line.split('\t')[9])
				return readLength

	def cleanReads(self):
		""" Clean reads """
		cleanReads = 0
		for line in self.sam:
			if line !='' and not self.patternAnnotation.match(line):
				cleanReads += 1
		return cleanReads

	def cleanBases(self):
		""" Clean bases """
		readLength = self.readLength()
		cleanBases = 0
		for line in self.sam:
			if line != '' and not self.patternAnnotation.match(line):
				cleanBases += readLength
		return cleanBases
				
	def depth(self):
		""" Bam deepth """
		pass

	def get_insersize(self):
		pass


	def extract(self,chrName):
		""" extract given ID name bam file """
		for line in self.sam:
			if line != '' and not self.patternAnnotation.match(line):
				chrID = line.split("\t")[2]
				if chrName == chrID:
					print (line)



#/usr/bin/python3

import os,sys
import re
import gzip
from collections import defaultdict

"""
Author: Zhu Sitao
Date : 2018-3-21
Dest: a fastq class; used in python 3.6
"""

RANGES = {
    'sanger': (33, 75),
    'illumina-1.8': (33, 79),
    'solexa': (59, 106),
    'phred64': (64, 106),
}

class fastq_record(object):
    """a class for fastq record
    """
    def __init__(self,name,seq,desc,qual):
        self.read_name = name
        self.seq = seq
        self.desc = desc
        self.qual = qual

    def read_id(self):
        return self.read_name
    def seq(self):
        return self.seq
    def desc(self):
        return self.desc
    def qual(self):
        return self.qual
    def fastq_parse(self):
        """ parse to a list """
        return '\n'.join([self.read_name,self.seq,self.desc,self.qual])

class Fastq(object):
    """a class deal fastq file
        new fastq name: @HISEQ:310:C5MH9ANXX:1:1101:3517:2043 2:N:0:TCGGTCAC
        old fastq name: @FCD1PB1ACXX:4:1101:1799:2201#GAAGCACG/2
    """
    def __init__(self,path):
        self.path = path
        self.patternNew = re.compile(r'(?P<id>\d):N:(\d):(?P<index>\S+)',re.VERBOSE)
        self.patternOld = re.compile(r'\@\S+\#(?P<index>\S+?)\/(?P<id>\d)',re.VERBOSE)

    def readFastq(self):
        """ Return fastq file line by line """
        if self.path.endswith("gz"):
            FH = gzip.open(self.path,'r')
        else:
            FH = open(self.path, 'r')
        for line in FH:
            yield line
        FH.close()
    def fastq_to_dict(self):
        """ return dict{fastq_id:}"""
        fqdict = {}
        flag = 0
        record = []
        for line in self.readFastq():
            line = line.strip()
            flag += 1
            if flag == 1:
                record.append(line)  # ID
            if flag == 2:
                record.append(line)  # sequence
            if flag == 3:
                record.append(line)  # description
            if flag == 4:
                record.append(line)  # quality
                fqdict[record[0]] = fastq_record(record[0],record[1],record[2],record[3])
                record.clear() # clear list
                flag = 0       # break
        return fqdict


    def qualitySystem(self):
        """ Return quality system """
        lineCount = 0
        checklines = ''
        for line in self.readFastq():
            lineCount += 1
            if lineCount > 100:
                break
            else:
                if lineCount % 4 == 0:
                    checklines += line.strip()
        c = [ord(x) for x in checklines]
        mi,ma = min(c),max(c)
        for format,v in RANGES.items():
            m1,m2 = v
            if mi >= m1 and ma <= m2:
                return format
        MAX = 0
        MIN = 100
        for one in map(ord,checklines):
            if one > MAX:
                MAX = one
            if one < MIN:
                MIN = one
        if MAX <= 74 and MIN < 59:
            # sanger and Ilumina 1.8+
            return "Phred+33"
        elif MAX > 73 and MIN >=64:
            # Illumina 1.3+ and Illumina 1.5+
            return "Phred+64"
        elif MAX >73 and MIN >= 59 and MIN < 64:
            # Solexa+64
            return "Solexa+64"
        else:
            return "Unknown score encoding"

    def qualsFormat(self):
        """ Return quality system """
        lineCount = 0
        checklines = ''
        for line in self.readFastq():
            lineCount += 1
            if lineCount > 100:
                break
            else:
                if lineCount % 4 == 0:
                    checklines += line.strip()
        c = [ord(x) for x in checklines]
        mi,ma = min(c),max(c)
        for format,v in RANGES.items():
            m1,m2 = v
            if mi >= m1 and ma <= m2:
                return format

    def indexSequence(self):
        """ Return index list """
        indexDict = defaultdict(int)
        indexList = []
        for line in self.readFastq():
            matchNew = self.patternNew.search(line)
            matchOld = self.patternOld.search(line)
            if matchNew:
                indexSequence = matchNew.group("index")
                indexDict[indexSequence] += 1
            if matchOld:
                indexSequence = matchOld.group("index")
                indexDict[indexSequence] += 1
        for key in indexDict.keys():
            indexList.append(key)
        return indexList

    def pairOrSingel(self):
        """ Return the fastq type: single end or paired ends """
        count, singel, pair = 0, 0, 0
        for line in self.readFastq():
            matchNew = self.patternNew.search(line)
            matchOld = self.patternOld.search(line)
            if matchNew:
                count += 1
                if matchNew.group("id") == "1":
                    singel += 1
                else:
                    pair += 1
            if matchOld:
                count += 1
                if matchOld.group("id") == '1':
                    singel += 1
                else:
                    pair += 1

        if count == singel:
            return "singel end"
        if count == pair :
            return "pair end"


    def to_fasta(self,output_fasta):
        """ Transformate fastq into fasta """
        OUT=open(output_fasta,'w')
        flag = 0
        for line in self.readFastq():
            flag += 1
            if flag == 1:
                OUT.writelines(">"+line) ## ID
            if flag == 2:
                OUT.writelines(line) ## sequence
            if flag == 4:
                flag = 0             ## break
        OUT.close()







# -*- coding: UTF-8 -*-
import os

def readme():
    with open("README.md") as f:
        return f.read()

from setuptools import setup 
setup(
    name='promoterExtract',
    version='0.2',
    keywords='promoter',
    description='Extract promoter sequence for biologists',
    long_description_content_type='str',
    long_description=readme(),
    author='zhusitao',
    author_email='zhusitao1990@163.com',
    url='https://github.com/zhusitao1990',
    include_package_data=True,
    packages=['promoterExtract'],
    license='MIT',
    install_requires = ['requests'])



#coding:utf-8
import os
import sys
import argparse

"""
https://genome.ucsc.edu/FAQ/FAQformat.html#format5
The multiple alignment format stores a series of multiple alignments in a format that is easy to parse and relatively easy to read
MAF格式通常用于记录体细胞突变。

MAF本来并不是一个常见的文本文件格式，只是因为癌症研究实在是太热门了，对它的理解也变得需求旺盛起来了。

一、MAF的说明

这些文件应该使用下面描述的突变注释格式（MAF）进行格式化。另外下文中有文件命名规范。

以下几种类型的体细胞突变会在MAF文件中出现：

错义突变及无义突变
剪接位点，其定义为剪接位点2 bp以内的SNP
沉默突变
与基因的编码区、剪接位点或遗传元件目标区域重叠的引物。
移码突变
*调控区突变
大部分MAF提交提交的是原始数据。这些原始数据中在体细胞中标记的位点与已知的变异类型相重合的。为避免有可能出现的细胞系污染，MAF规定了一定的下细胞过滤标准。根据现行政策，可开放获取MAF资料应满足：

包括所有已验证的体细胞突变名称
包括与编码区域或剪接位点重叠的所有未验证的体细胞突变名称
排除所有其他类型的突变（即非体细胞突变、不在编码区域或剪接位点的未验证体的细胞突变以及未在dbSNP、COSMIC或OMIM中注释为体细胞的dbSNP位点）
我们提交给DCC MAF存档的数据包括两种：Somatic MAF（named .somatic.maf）的开放访问数据以及不经过筛选的包含原始数据的Protected MAF（named.protected.maf）。所有数据将使用MAF标准进行格式化。
MAF文件可能有两种格式 ，可能是47列，或者120列，第一行一般都是 头文件，注释着每一列的信息，的确，信息量有点略大。如下：
     1  Hugo_Symbol
     2  Entrez_Gene_Id
     3  Center
     4  NCBI_Build
     5  Chromosome
     6  Start_Position
     7  End_Position
     8  Strand
     9  Consequence
    10  Variant_Classification
    11  Variant_Type
    12  Reference_Allele
    13  Tumor_Seq_Allele1
    14  Tumor_Seq_Allele2
    15  dbSNP_RS
    16  dbSNP_Val_Status
    17  Tumor_Sample_Barcode
    18  Matched_Norm_Sample_Barcode
    19  Match_Norm_Seq_Allele1
    20  Match_Norm_Seq_Allele2
    21  Tumor_Validation_Allele1
    22  Tumor_Validation_Allele2
    23  Match_Norm_Validation_Allele1
    24  Match_Norm_Validation_Allele2
    25  Verification_Status
    26  Validation_Status
    27  Mutation_Status
    28  Sequencing_Phase
    29  Sequence_Source
    30  Validation_Method
    31  Score
    32  BAM_File
    33  Sequencer
    34  t_ref_count
    35  t_alt_count
    36  n_ref_count
    37  n_alt_count
    38  HGVSc
    39  HGVSp
    40  HGVSp_Short
    41  Transcript_ID
    42  RefSeq
    43  Protein_position
    44  Codons
    45  Hotspot
    46  cDNA_change
    47  Amino_Acid_Change
     1  Hugo_Symbol
     2  Entrez_Gene_Id
     3  Center
     4  NCBI_Build
     5  Chromosome
     6  Start_Position
     7  End_Position
     8  Strand
     9  Variant_Classification
    10  Variant_Type
    11  Reference_Allele
    12  Tumor_Seq_Allele1
    13  Tumor_Seq_Allele2
    14  dbSNP_RS
    15  dbSNP_Val_Status
    16  Tumor_Sample_Barcode
    17  Matched_Norm_Sample_Barcode
    18  Match_Norm_Seq_Allele1
    19  Match_Norm_Seq_Allele2
    20  Tumor_Validation_Allele1
    21  Tumor_Validation_Allele2
    22  Match_Norm_Validation_Allele1
    23  Match_Norm_Validation_Allele2
    24  Verification_Status
    25  Validation_Status
    26  Mutation_Status
    27  Sequencing_Phase
    28  Sequence_Source
    29  Validation_Method
    30  Score
    31  BAM_File
    32  Sequencer
    33  Tumor_Sample_UUID
    34  Matched_Norm_Sample_UUID
    35  HGVSc
    36  HGVSp
    37  HGVSp_Short
    38  Transcript_ID
    39  Exon_Number
    40  t_depth
    41  t_ref_count
    42  t_alt_count
    43  n_depth
    44  n_ref_count
    45  n_alt_count
    46  all_effects
    47  Allele
    48  Gene
    49  Feature
    50  Feature_type
    51  One_Consequence
    52  Consequence
    53  cDNA_position
    54  CDS_position
    55  Protein_position
    56  Amino_acids
    57  Codons
    58  Existing_variation
    59  ALLELE_NUM
    60  DISTANCE
    61  TRANSCRIPT_STRAND
    62  SYMBOL
    63  SYMBOL_SOURCE
    64  HGNC_ID
    65  BIOTYPE
    66  CANONICAL
    67  CCDS
    68  ENSP
    69  SWISSPROT
    70  TREMBL
    71  UNIPARC
    72  RefSeq
    73  SIFT
    74  PolyPhen
    75  EXON
    76  INTRON
    77  DOMAINS
    78  GMAF
    79  AFR_MAF
    80  AMR_MAF
    81  ASN_MAF
    82  EAS_MAF
    83  EUR_MAF
    84  SAS_MAF
    85  AA_MAF
    86  EA_MAF
    87  CLIN_SIG
    88  SOMATIC
    89  PUBMED
    90  MOTIF_NAME
    91  MOTIF_POS
    92  HIGH_INF_POS
    93  MOTIF_SCORE_CHANGE
    94  IMPACT
    95  PICK
    96  VARIANT_CLASS
    97  TSL
    98  HGVS_OFFSET
    99  PHENO
   100  MINIMISED
   101  ExAC_AF
   102  ExAC_AF_Adj
   103  ExAC_AF_AFR
   104  ExAC_AF_AMR
   105  ExAC_AF_EAS
   106  ExAC_AF_FIN
   107  ExAC_AF_NFE
   108  ExAC_AF_OTH
   109  ExAC_AF_SAS
   110  GENE_PHENO
   111  FILTER
   112  CONTEXT
   113  src_vcf_id
   114  tumor_bam_uuid
   115  normal_bam_uuid
   116  case_id
   117  GDC_FILTER
   118  COSMIC
   119  MC3_Overlap
   120  GDC_Validation_Status
"""
class maf(object):
    """ a class for maf """
    def __init__(self):
        pass
